@@ -170,7 +170,14 @@ def get_pnl_plot(delta, ret, fig_return=False, alpha=True, **kwargs):
     day_pnl_short = pnl_short.groupby(by=pnl_short.index.to_series().apply(lambda x: x.date())).sum()
     DD_ss = day_pnl.cumsum() - day_pnl.cumsum().expanding(0).max()
 
-    day_pnl.cumsum().plot(ax=ax1, label='Strategy')
+    #
+    if 'title' in kwargs:
+        temp = kwargs.pop('title')
+        s_label = f'Strategy: {temp}'
+    else:
+        s_label = 'Strategy'
+
+    day_pnl.cumsum().plot(ax=ax1, label=s_label)
     day_pnl_long.cumsum().plot(ax=ax1, label='Only Long', c='r', ls='--')
     day_pnl_short.cumsum().plot(ax=ax1, label='Only Short', c='g', ls='--')
     day_ret.cumsum().plot(ax=ax1, ls=':', c='k', alpha=0.5, label='Asset')
@@ -314,7 +321,13 @@ def get_composite_pnl_plot(delta, ret, fig_return=False, alpha=True, **kwargs):
     DD_ss = day_pnl.cumsum() - day_pnl.cumsum().expanding(0).max()
 
     #
-    day_pnl.cumsum().plot(ax=ax1, label='Strategy')
+    if 'title' in kwargs:
+        temp = kwargs.pop('title')
+        s_label = f'Strategy: {temp}'
+    else:
+        s_label = 'Strategy'
+
+    day_pnl.cumsum().plot(ax=ax1, label=s_label)
     day_pnl_long.cumsum().plot(ax=ax1, label='Only Long', c='r', ls='--')
     day_pnl_short.cumsum().plot(ax=ax1, label='Only Short', c='g', ls='--')
     day_ret.cumsum().plot(ax=ax1, ls=':', c='k', alpha=0.5, label='Asset')
