@@ -24,6 +24,19 @@ def _preprocessing(x, y):
     return factor, ret
 
 
+def _preprocessing_cs(x, y):
+    x = copy(x)
+    y = copy(y)
+
+    factor = pd.DataFrame(x)
+    ret = pd.DataFrame(y)
+
+    factor[np.isinf(factor)] = np.nan
+    ret[np.isinf(ret)] = np.nan
+
+    return factor, ret
+
+
 def _get_report(delta, ret):
     """
     :param delta: pd.Series
