@@ -5,7 +5,7 @@
 import pandas as pd
 import numpy as np
 import matplotlib.pyplot as plt
-from .private_func import _preprocessing, _corr2
+from .private_func import _preprocessing, _corr2, _preprocessing_cs
 import warnings
 from .backtest import get_pnl_plot
 
@@ -103,7 +103,7 @@ class simpletest():
                 output['delta_sign'] = indicator1
 
         if val:
-            indicator2 = factor/factor.abs().mean() * 100  * np.sign(corr)
+            indicator2 = factor / factor.std() * 100 * np.sign(corr)
             label2 = 'Val-Trade: '
 
             pnl2 = ret * indicator2
@@ -279,6 +279,94 @@ class simpletest():
 
         return
 
+    def simple_pnl_cs(self, x, y, dn=False, ax=None, horizon=3, **kwargs):
+        if len(x) != len(y):
+            print(F'mismatched length of x, y, len(x) == {len(x)} len(y) == {len(y)}')
+            return
+
+        if not (isinstance(x, pd.DataFrame)) or not (isinstance(y, pd.DataFrame)):
+            print('x and y must be pd.DataFrame')
+            return
+
+        if not (isinstance(x.index, pd.core.indexes.datetimes.DatetimeIndex)) \
+                or not (isinstance(y.index, pd.core.indexes.datetimes.DatetimeIndex)):
+            print('index of x and y must be pd.core.indexes.datetimes.DatetimeIndex')
+            return
+
+        factor, ret = _preprocessing_cs(x, y)
+
+        # corr = _corr2(factor.stack(dropna=False), ret.stack(dropna=False))
+        # factor = corr * factor
+
+        def factor_normalize(factor_og):
+            factor = factor_og.copy()
+
+            #
+            factor[np.isinf(factor)] = np.nan
+            factor = factor.apply(lambda m: (m - m.mean()) / m.std(), axis=1)
+            factor[factor > 4.0] = 4.0
+            factor[factor < -4.0] = -4.0
+            factor = factor.apply(lambda m: (m - m.mean()) / m.std(), axis=1)
+
+            return factor
+
+        def factor_2_delta(factor_og):
+            factor = factor_og.copy()
+            if dn:
+                factor = factor_normalize(factor)
+                #
+                delta = factor.apply(lambda m: m / m.abs().sum(), axis=1) * 100.0
+            else:
+                delta = factor.apply(lambda m: m / m.std(), axis=0).multiply(100/factor.count(axis=1), axis=0)
+
+            return delta
+
+        if ax is None:
+            fig = plt.figure(figsize=(15, 6))
+            ax = fig.add_subplot(111)
+
+        delta = factor_2_delta(factor)
+        delta_bench = pd.DataFrame(100.0, index=delta.index, columns=delta.columns).div(delta.count(axis=1), axis=0)
+        delta_bench[np.isinf(delta_bench)] = np.nan
+        delta_bench[np.isnan(delta)] = np.nan
+
+        #
+        pnl_bench = (delta_bench * ret).sum(axis=1)
+        day_pnl_bench = pnl_bench.groupby(by=pnl_bench.index.date).sum()
+        sharpe = round(day_pnl_bench.mean() / day_pnl_bench.std() * 16, 2)
+        pnl_y = round(day_pnl_bench.mean() * 251, 2)
+        day_pnl_bench.cumsum().plot(ax=ax, label='Asset Cumulative Return: PnL={:.2f} '
+                                                 'Sharpe={:.2f}'.format(pnl_y, sharpe),
+                                    ls=':', c='k', alpha=0.5)
+
+        #
+        for i in range(horizon):
+            pnl = (delta * ret.shift(-i)).sum(axis=1)
+            day_pnl = pnl.groupby(by=pnl.index.date).sum()
+
+            # report
+            sharpe = round(day_pnl.mean() / day_pnl.std() * 16, 2)
+
+            pnl_y = round(day_pnl.mean() * 251, 2)
+
+            pot = round(pnl.sum() / (delta.diff().abs().sum().sum()) * 1e4, 2)
+
+            #
+            day_pnl.cumsum().plot(ax=ax,
+                                  label=f'Lag{i}: ' + 'PnL={:.2f} Sharpe={:.2f} PoT={:.2f}'.format(pnl_y, sharpe, pot))
+
+        if 'title' in kwargs:
+            title = kwargs.pop('title')
+            ax.set_title(f'Simple PnL (info: {str(title)})')
+        else:
+            ax.set_title('Simple PnL')
+
+        ax.set_ylabel('PnL')
+        ax.set_xlabel('Date')
+        ax.legend()
+
+        return
+
     def plot_cdf(self, x, y,
                  ax=None, ax_twinx=None, with_legend=True,
                  normalize_y=False):
@@ -486,9 +574,9 @@ class simpletest():
 
         rank_ic = x.rolling(window).apply(_rolling_ic)
 
-        return rank_ic.mean()/rank_ic.std()
+        return rank_ic.mean() / rank_ic.std()
 
-    def plot_composite(self, x, y, markout_periods=30, cdf_period2=5, fig_return=False,
+    def plot_composite(self, x, y, markout_periods=20, cdf_period2=5, fig_return=False,
                        **kwargs):
         """
         x, y: prefer pd.Series
@@ -573,3 +661,71 @@ class simpletest():
             return fig
 
         return
+
+    def plot_composite_cs(self, x, y,
+                          markout_periods=20, cdf_period2=5, fig_return=False, horizon=3,
+                          **kwargs):
+        """"""
+        if len(x) != len(y):
+            print(F'mismatched length of x, y, len(x) == {len(x)} len(y) == {len(y)}')
+            return
+
+        if not (isinstance(x, pd.DataFrame)) or not (isinstance(y, pd.DataFrame)):
+            print('x and y must be pd.DataFrame')
+            return
+
+        if not (isinstance(x.index, pd.core.indexes.datetimes.DatetimeIndex)) \
+                or not (isinstance(y.index, pd.core.indexes.datetimes.DatetimeIndex)):
+            print('index of x and y must be pd.core.indexes.datetimes.DatetimeIndex')
+            return
+
+        x, y = _preprocessing_cs(x, y)
+
+        if 'ic' in kwargs:
+            if kwargs.pop('ic'):
+                print('IC: ', self.ic(x.stack(), y.stack()))
+
+        if 'figsize' in kwargs:
+            figsize = kwargs.pop('figsize')
+        else:
+            figsize = (16, 24)
+
+        fig = plt.figure(figsize=figsize)
+
+        # cdf
+        ax1 = plt.subplot2grid((3, 2), (0, 0), fig=fig)
+        self.plot_cdf(x.stack(dropna=False), y.stack(dropna=False), ax=ax1)
+        ax1.set_title('CDF 1Period')
+
+        ax2 = plt.subplot2grid((3, 2), (0, 1), fig=fig)
+        self.plot_cdf(x.stack(dropna=False),
+                      y.shift(1).rolling(cdf_period2).sum().shift(-cdf_period2).stack(dropna=False), ax=ax2)
+        ax2.set_title('CDF {}Period'.format(cdf_period2))
+
+        # markout
+        Y = pd.concat([y.rolling(i).sum().shift(-i + 1).stack(dropna=False).rename(str(i)) \
+                       for i in range(1, markout_periods + 1)], axis=1)
+
+        ax3 = plt.subplot2grid((3, 2), (1, 0), fig=fig)
+        self.plot_markout_quantile(x.stack(dropna=False), Y, ax=ax3, islegend=True)
+        ax3.set_title('Markout')
+        ax3.legend()
+
+        # quantile true ret
+        ax4 = plt.subplot2grid((3, 2), (1, 1), fig=fig)
+        self.plot_quantile_ret(x.stack(dropna=False), y.stack(dropna=False), ax=ax4)
+        ax4.set_title('Quantile Ret Mean')
+
+        # simple pnl cs
+        ax5 = plt.subplot2grid((3, 2), (2, 0), colspan=2, fig=fig)
+        if 'title' in kwargs:
+            title = kwargs.pop('title')
+        else:
+            title = ''
+
+        self.simple_pnl_cs(x, y, ax=ax5, horizon=horizon, title=title)
+
+        if fig_return:
+            return fig
+
+        return
